<?php
/**
 * @brief		Featured Content Application Class
 * @author		<a href='http://ipsviet.com'>onlyME</a>
 * @copyright	(c) 2014 onlyME
 * @package		IPS Social Suite
 * @subpackage	Featured Content
 * @since		11 Dec 2014
 * @version		
 */
 
namespace IPS\fcontent;

/**
 * Featured Content Application Class
 */
class _Application extends \IPS\Application
{
	/**
	 * Install 'other' items. Left blank here so that application classes can override for app
	 *  specific installation needs. Always run as the last step.
	 *
	 * @return void
	 */
	public function installOther()
	{
		if ( \IPS\Application::appIsEnabled( 'featuredcontent' ) )
		{
			foreach ( \IPS\Db::i()->select( '*', 'featuredcontent_sliders' ) as $row )
			{
				foreach( array_keys($row) as $k )
				{
					if ( $k != 'fcs_id' )
					{
						if( \IPS\Db::i()->checkForColumn( 'fcontent_sliders', $k ) )
						{
							$sliders[$k] = $row[$k];
						}
					}
				}
				$new = \IPS\Db::i()->insert( 'fcontent_sliders', $sliders );
				
				if ( $row['fcs_method'] == 'noauto' )
				{
					foreach ( \IPS\Db::i()->select( '*', 'featuredcontent_contents', array( 'fcc_slider=?', $row['fcs_id'] ) ) as $con )
					{
						foreach( array_keys($con) as $cK )
						{
							if ( $cK != 'fcc_id' )
							{
								if( \IPS\Db::i()->checkForColumn( 'fcontent_contents', $cK ) )
								{
									$contents['fcc_slider'] = $new;
									$contents[$cK] = $con[$cK];
								}
							}
						}
						if ( \count( $contents ) > 0 )
						{
							$newContents[] = $contents;
						}
					}
					
					if ( \count( $newContents ) > 0 )
					{
						foreach( $newContents as $nCon )
						{
							\IPS\Db::i()->insert( 'fcontent_contents', $nCon );
						}
					}
				}
			}
			
			# Clear all uploaded image from database of the old app to prevent missing images when uninstalling iterator_apply
			\IPS\Db::i()->update( 'featuredcontent_contents', array( 'fcc_uploadimg' => NULL ), array() );
						
			# Update permissions
			\IPS\Db::i()->update( 'core_permission_index', array( 'app' => 'fcontent' ), array( 'app=?', 'featuredcontent' ) );
			
			# Update widgets
			\IPS\Db::i()->update( 'core_widget_areas', "widgets=REPLACE(widgets, 'featuredcontent', 'fcontent' ), widgets=REPLACE(widgets, 'fcontentWidget', 'fcWidget' )" );
			if ( \IPS\Db::i()->checkForTable('cms_page_widget_areas') )
			{
				\IPS\Db::i()->update( 'cms_page_widget_areas', "area_widgets=REPLACE(area_widgets, 'featuredcontent', 'fcontent' ), area_widgets=REPLACE(area_widgets, 'fcontentWidget', 'fcWidget' )" );
			}
			if ( \IPS\Db::i()->checkForTable('custompages_page_widget_areas') )
			{
				\IPS\Db::i()->update( 'custompages_page_widget_areas', "area_widgets=REPLACE(area_widgets, 'featuredcontent', 'fcontent' ), area_widgets=REPLACE(area_widgets, 'fcontentWidget', 'fcWidget' )" );
			}
			
			# Disable old app
			try
			{
				$app = \IPS\Application::load('featuredcontent');
				$app->_enabled = 0;
				$app->save();				
			}
			catch ( \OutOfRangeException $e ){}
		
			# Clear site cache
			\IPS\Output::clearJsFiles();
			\IPS\Theme::deleteCompiledCss();
			foreach( \IPS\Theme::themes() as $id => $set )
			{
				$set->cache_key = md5( microtime() . mt_rand( 0, 1000 ) );
				$set->save();
			}			
			\IPS\Data\Store::i()->clearAll();
			\IPS\Data\Cache::i()->clearAll();			
		}
		
		$path = \IPS\Application::getRootPath() . '/uploads/fcontentCache/';
		
		if ( file_exists( $path ) )
		{
			foreach ( new \DirectoryIterator( $path ) as $file )
			{
				if ( $file->isDot() )
				{
					continue;
				}
				@unlink( $file->getPathname() );
			}		
		}		
	}
	
	/**
	 * [Node] Get Icon for tree
	 *
	 * @note	Return the class for the icon (e.g. 'globe')
	 * @return	string|null
	 */
	protected function get__icon()
	{
		return 'star';
	}
	
	public static $loadedJSandCSS = false;
	
	public static function show( $id, $conf=NULL )
	{
		if ( ! self::$loadedJSandCSS )
		{
			self::$loadedJSandCSS = true;
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_slider.js', 'fcontent', 'front' ) );
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'slider.css', 'fcontent', 'front' ) );
		}
			
		try
		{	
			$slider = \IPS\fcontent\Slider::loadAndCheckPerms( $id, 'view' );
			
			if ( $slider->enabled != 1 || !$slider->getContents() )
			{
				return \IPS\Theme::i()->getTemplate( 'embed', 'fcontent', 'front' )->noContent( $slider );
			}
			
			if ( $slider->style == 'grid' )
			{
				$html = \IPS\Theme::i()->getTemplate( 'embed', 'fcontent', 'front' )->showGrid( $slider, $slider->getContents(), $conf );
			}
			else
			{
				$html = \IPS\Theme::i()->getTemplate( 'embed', 'fcontent', 'front' )->showSlider( $slider, $slider->getContents(), $conf );
			}
			
			return "<ul class='ipsList_reset'>" . $html . "</ul>";
		}
		catch( \OutOfRangeException $e )
		{
			return '';
		}			
	}
	
	public static function easingList()
	{
		$data = array(  'none', 'swing', 'easeInQuad', 'easeOutQuad', 'easeInOutQuad', 'easeInCubic', 'easeOutCubic', 'easeInOutCubic', 'easeInQuart', 'easeOutQuart',
						'easeInOutQuart', 'easeInQuint', 'easeOutQuint', 'easeInOutQuint', 'easeInSine', 'easeOutSine', 'easeInOutSine', 'easeInExpo', 'easeOutExpo',
						'easeInOutExpo', 'easeInCirc', 'easeOutCirc', 'easeInOutCirc', 'easeInElastic', 'easeOutElastic', 'easeInOutElastic', 'easeInBack',
						'easeOutBack', 'easeInOutBack', 'easeInBounce', 'easeOutBounce', 'easeInOutBounce' );

		foreach( $data as $v )
		{
			$easing[$v] = $v;
		}
		
		return $easing;
	}	
}