<?php
/**
 * @brief		Background Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Featured Content Pro
 * @since		10 Aug 2019
 */

namespace IPS\fcontent\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _clearCache
{
	
	/**
	 * @brief Number of topics to build per cycle
	 */
	public $rebuild = 20;
	
	/**
	 * Parse data before queuing
	 *
	 * @param	array	$data
	 * @return	array
	 */
	public function preQueueData( $data )
	{
		# Count
		try
		{
			$data['count']		= \IPS\Db::i()->select( 'MAX(id)', 'fcontent_cache' )->first();
			$data['realCount']	= \IPS\Db::i()->select( 'COUNT(*)', 'fcontent_cache' )->first();
		}
		catch( \Exception $ex )
		{
			return NULL;
		}
		
		if( $data['count'] == 0 or $data['realCount'] == 0 )
		{
			return NULL;
		}

		$data['indexed'] = 0;
		
		return $data;
	}

	/**
	 * Run Background Task
	 *
	 * @param	mixed						$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int							$offset	Offset
	 * @return	int							New offset
	 * @throws	\IPS\Task\Queue\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function run( $data, $offset )
	{
		$count = 0;
		
		$select = \IPS\Db::i()->select( '*', 'fcontent_cache', NULL, 'id', array( 0, $this->rebuild ) );
		
		foreach( $select as $row )
		{
			try
			{
				\IPS\File::get( 'fcontent_Cache', $row['cache'] )->delete();
			}
			catch( \Exception $e ){}
			
			$delete[] = $row['id'];
			$data['indexed']++;
			$last = $row['id'];				
		}	
		
		if ( $delete && \count( $delete ) > 0 )
		{
			\IPS\Db::i()->delete( 'fcontent_cache', array( 'id IN (?)', implode( ",", $delete ) ) );
		}			
		
		# No last
		if( $last === NULL )
		{
			throw new \IPS\Task\Queue\OutOfRangeException;
		}
		
		# Done
		return $last;		
	}
	
	/**
	 * Get Progress
	 *
	 * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
	 * @param	int						$offset	Offset
	 * @return	array( 'text' => 'Doing something...', 'complete' => 50 )	Text explaining task and percentage complete
	 * @throws	\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
	 */
	public function getProgress( $data, $offset )
	{
		return array( 'text' => \IPS\Member::loggedIn()->language()->addToStack('fcontent_queue_delete_cache'), 'complete' => ( $data['realCount'] * $data['indexed'] ) > 0 ? round( ( $data['realCount'] * $data['indexed'] ) * 100, 2 ) : 0 );
	}

	/**
	 * Perform post-completion processing
	 *
	 * @param	array	$data
	 * @return	void
	 */
	public function postComplete( $data )
	{

	}
}