<?php

namespace IPS\fcontent;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Featured Content Node
 */
class _Slider extends \IPS\Node\Model implements \IPS\Node\Permissions
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
		
	/**
	 * @brief	[ActiveRecord] Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'fcontent_sliders';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'fcs_';
		
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'fc_slider_sliders';
			
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = FALSE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @encode
	 */
	protected static $restrictions = array(
		'app'		=> 'fcontent',
		'module'	=> 'sliders',
		'prefix' 	=> 'sliders_',
		'all'		=> 'sliders_manage',
	);
	
	/** 
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'fcontent';
	
	/** 
	 * @brief	[Node] Type for permission index
	 */
	public static $permType = 'slider';
	
	/**
	 * @brief	The map of permission columns
	 */
	public static $permissionMap = array(
		'view' 		=> 'view',
		'manage'	=> 3,
	);

	/** 
	 * @brief	[Node] Moderator Permission
	 */
	public static $modPerm = 'slider_sliders';
	
	
	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'fcperm_';
	
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->enabled;
	}

	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->enabled	= $enabled;
	}	
	
	/**
	 * [Node] Get Node Title
	 *
	 * @return	string
	 */
	protected function get__title()
	{
		return $this->title;
	}
	
	protected function get__description()
	{
		$code = "{fcontent=&quot;" . $this->id . "&quot;}";
		return $code;
	}
	
	protected function get_titleHeight()
	{
		$default = 35 + 20;
		$line = $this->truncate > 0 ? $this->truncate : 1;
		if ( $line == 1 )
		{
			$height = $default;
		}
		else
		{
			$height = $default + ($line-1) * 18;
		}
		return $height . 'px';
	}	
	
	protected function get_itemWidth()
	{
		return $this->img_w > 0 ? $this->img_w . "px" : 'auto';
	}	
	
	protected function get_itemHeight()
	{
		return $this->img_h > 0 ? $this->img_h . "px" : 'auto';
	}

	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		# Basic
		$form->addTab( 'fcs_tab_basic' );	
		$form->addHeader( 'fcs_tab_basic' );	
		$form->add( new \IPS\Helpers\Form\Text( 'fcs_title', $this->id ? $this->title : "", TRUE, array( 'maxLength' => 255 ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_showname', $this->id ? $this->showname : 0, FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_showtitle', $this->id ? $this->showtitle : 1, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_title_pos', $this->id ? $this->title_pos : "inside", FALSE, array(
			'options' => array(
				'inside' 	=> 'fcs_title_pos_inside',
				'below' 	=> 'fcs_title_pos_below'
			) ) ) );		
			
		$form->addHeader( 'fcs_tab_images' );
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_img_w', $this->id ? $this->img_w : 310, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_img_h', $this->id ? $this->img_h : 220, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_margin', $this->id ? $this->margin : 3, FALSE ) );
		$form->add( new \IPS\Helpers\Form\Upload( 'fcs_noimg', $this->noimg ? \IPS\File::get( 'core_Theme', $this->noimg ) : NULL, FALSE, array( 'image' => true, 'storageExtension' => 'core_Theme' ) ) );
		
		# Slider
		$form->addTab( 'fcs_tab_slider' );
		$sliderOpts = array( 'fcs_truncate', 'fcs_shownav', 'fcs_showpage', 'fcs_keyboard', 'fcs_ticker', 'fcs_maxSlides', 'fcs_minSlides', 'fcs_speed', 'fcs_duration', 'fcs_easing', 'fcs_autoplay', 'fcs_autocontrol' );
		
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_style', $this->id ? $this->style : "horizontal", FALSE, array(
			'options' => array(
				'horizontal' 	=> 'fcs_style_h',
				'vertical' 		=> 'fcs_style_v',
				'fade' 			=> 'fcs_style_fade',
				'grid' 			=> 'fcs_style_grid',
			),
			'toggles'	=> array(
				'horizontal' 	=> $sliderOpts,
				'vertical' 		=> $sliderOpts,
				'fade' 			=> $sliderOpts,			
			),
		) ) );
			
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_truncate', $this->id ? $this->truncate : 1, FALSE, array( 'min' => 1 ), NULL, NULL, NULL, 'fcs_truncate' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_shownav', $this->id ? $this->shownav : 1, FALSE, array(), NULL, NULL, NULL, 'fcs_shownav' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_showpage', $this->id ? $this->showpage : 1, FALSE, array(), NULL, NULL, NULL, 'fcs_showpage' ) );			
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_keyboard', $this->id ? $this->keyboard : 1, FALSE, array(), NULL, NULL, NULL, 'fcs_keyboard' ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_ticker', $this->id ? $this->ticker : 0, FALSE, array( 'togglesOff' => array( 'fcs_easing', 'fcs_autoplay', 'fcs_autocontrol', 'fcs_maxSlides', 'fcs_minSlides', 'fcs_duration' ) ), NULL, NULL, NULL, 'fcs_ticker' ) );		
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_maxSlides', ($this->id and $this->maxSlides > 0) ? $this->maxSlides : 5, FALSE, array(), NULL, NULL, NULL, 'fcs_maxSlides' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_minSlides', ($this->id and $this->minSlides > 0) ? $this->minSlides : 1, FALSE, array(), NULL, NULL, NULL, 'fcs_minSlides' ) );		
		
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_speed', ($this->id and $this->speed > 0) ? $this->speed : 500, FALSE, array(), NULL, NULL, NULL, 'fcs_speed' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_duration', ($this->id and $this->duration > 0) ? $this->duration : 3000, FALSE, array(), NULL, NULL, NULL, 'fcs_duration' ) );		
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_easing', $this->id ? $this->easing : "none", FALSE, array( 'options' => \IPS\Application::load('fcontent')->easingList() ), NULL, NULL, NULL, 'fcs_easing' ) );				
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_autoplay', $this->id ? $this->autoplay : 1, FALSE, array( 'togglesOn' => array( 'fcs_autocontrol' ) ), NULL, NULL, NULL, 'fcs_autoplay' ) );			
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_autocontrol', $this->id ? $this->autocontrol : 1, FALSE, array(), NULL, NULL, NULL, 'fcs_autocontrol' ) );
		
		# Contents
		$form->addTab( 'fcs_tab_content' );
		$form->addHeader( 'fcs_header_content' );	
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_total_items', ($this->id and $this->total_items > 0) ? $this->total_items : 20, FALSE, array(), NULL, NULL, NULL, 'fcs_total_items' ) );	
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_method', $this->id ? $this->method : "noauto", FALSE, array(
			'options' => array(
				'noauto' 	=> 'fcs_method_noauto',
				'cms' 		=> 'fcs_method_cms',				
				'forums' 	=> 'fcs_method_forum',
				'gallery' 	=> 'fcs_method_gallery',
				'downloads' => 'fcs_method_downloads',				
				'nexus' 	=> 'fcs_method_nexus',				
				'videobox' 	=> 'fcs_method_videobox',				
				'rss'		=> 'fcs_method_rss',
			),
			'toggles'	=> array(
				'cms'	=> array(
					'fcs_cms_db',
					'fcs_featured',
					'fcs_tags',
					'fcs_date_range',
				),			
				'forums'	=> array(
					'fcs_forums',
					'fcs_getlastimage',
					'fcs_skiptopicnoimg',
					'fcs_fromtthumb',
					'fcs_featured',
					'fcs_tags',
					'fcs_date_range',
				),
				'gallery'	=> array(
					'fcs_gallery_cat',
					'fcs_featured',
					'fcs_tags',
					'fcs_date_range',
				),
				'downloads'	=> array(
					'fcs_downloads_cat',
					'fcs_featured',
					'fcs_tags',
					'fcs_date_range',
				),
				'nexus'	=> array(
					'fcs_nexus_cat',
					'fcs_date_range',
				),
				'videobox'	=> array(
					'fcs_videobox_cat',
					'fcs_featured',
					'fcs_tags',
					'fcs_date_range',
				),					
				'rss'	=> array(
					'fcs_rssURL',
					'fcs_skiptopicnoimg',
					'fcs_tags',
					'fcs_date_range',
				),
			)
		) ) );
				
		/* Pages */
		if ( \IPS\Application::appIsEnabled( 'cms' ) )
		{			
			$fcs_cms_db = array();

			foreach( \IPS\cms\Databases::roots( NULL, NULL ) as $db )
			{
				$fcs_cms_db[ $db->id ] = $db->_title;
				$showCat[ $db->id ] = array( 'fcs_cms_cat_' . $db->id, 'fcs_cms_field_' . $db->id );
				$dbIDs[] = $db->id;
				\IPS\Member::loggedIn()->language()->words['fcs_cms_cat_' . $db->id] = "Categories from " . $db->_title; 
				\IPS\Member::loggedIn()->language()->words['fcs_cms_field_' . $db->id] = "Assign Image from Field"; 
				\IPS\Member::loggedIn()->language()->words['fcs_cms_field_' . $db->id . '_desc'] = "If not selected any field, the record  image will be priority"; 				
			}
		
			if ( !empty( $fcs_cms_db ) )
			{
				$form->add( new \IPS\Helpers\Form\Select( 'fcs_cms_db', explode( ',', $this->cms_db ), FALSE, array( 'options' => $fcs_cms_db, 'toggles' => $showCat, 'multiple' => FALSE ), NULL, NULL, NULL, 'fcs_cms_db' ) );				
				foreach( $dbIDs as $dbID )
				{
					$form->add( new \IPS\Helpers\Form\Node( 'fcs_cms_cat_' . $dbID, ( $this->id and $this->cms_cat != '*' ) ? $this->cms_cat : 0, FALSE, array( 'class' => '\IPS\cms\Categories' . $dbID, 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_cms_cat_' . $dbID ) );	
					$form->add( new \IPS\Helpers\Form\Select( 'fcs_cms_field_' . $dbID, $this->id ? $this->cms_fieldIMG : 0, FALSE, array( 'options' => $this->getFieldsFromDB($dbID), 'multiple' => FALSE  ), NULL, NULL, NULL, 'fcs_cms_field_' . $dbID ) );	
				}
			}
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_cms_db', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noCMS') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_cms_db' ) );
		}
		
		/* Forums */
		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{		
			$form->add( new \IPS\Helpers\Form\Node( 'fcs_forums', ( $this->id and $this->forums != '*' ) ? $this->forums : 0, FALSE, array( 'clubs' => true, 'class' => 'IPS\forums\Forum', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_forums' ) );	
			$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_getlastimage', $this->id ? $this->getlastimage : 0, FALSE, array(), NULL, NULL, NULL, 'fcs_getlastimage' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_skiptopicnoimg', $this->id ? $this->skiptopicnoimg : 0, FALSE, array(), NULL, NULL, NULL, 'fcs_skiptopicnoimg' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_fromtthumb', $this->id ? $this->fromtthumb : 0, FALSE, array(), NULL, NULL, NULL, 'fcs_fromtthumb' ) );
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_forums', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noForums') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_forums' ) );
		}
		
		/* Gallery */
		if ( \IPS\Application::appIsEnabled( 'gallery' ) )
		{		
			$form->add( new \IPS\Helpers\Form\Node( 'fcs_gallery_cat', ( $this->id and $this->gallery_cat != '*' ) ? $this->gallery_cat : 0, FALSE, array( 'clubs' => true, 'class' => 'IPS\gallery\Category', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_gallery_cat' ) );	
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_gallery_cat', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noGallery') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_gallery_cat' ) );
		}
		
		/* Downloads */
		if ( \IPS\Application::appIsEnabled( 'downloads' ) )
		{		
			$form->add( new \IPS\Helpers\Form\Node( 'fcs_downloads_cat', ( $this->id and $this->downloads_cat != '*' ) ? $this->downloads_cat : 0, FALSE, array( 'clubs' => true, 'class' => 'IPS\downloads\Category', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_downloads_cat' ) );	
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_downloads_cat', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noDownloads') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_downloads_cat' ) );
		}
		
		/* Nexus */
		if ( \IPS\Application::appIsEnabled( 'nexus' ) )
		{		
			$form->add( new \IPS\Helpers\Form\Node( 'fcs_nexus_cat', ( $this->id and $this->nexus_cat != '*' ) ? $this->nexus_cat : 0, FALSE, array( 'clubs' => false, 'class' => 'IPS\nexus\Package\Group', 'subnodes' => FALSE, 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_nexus_cat' ) );	
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_nexus_cat', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noNexus') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_nexus_cat' ) );
		}	
		
		/* Videobox */
		if ( \IPS\Application::appIsEnabled( 'videobox' ) )
		{		
			$form->add( new \IPS\Helpers\Form\Node( 'fcs_videobox_cat', ( $this->id and $this->videobox_cat != '*' ) ? $this->videobox_cat : 0, FALSE, array( 'clubs' => true, 'class' => 'IPS\videobox\Category', 'multiple' => TRUE, 'zeroVal' => 'all'  ), NULL, NULL, NULL, 'fcs_videobox_cat' ) );	
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Custom( 'fcs_videobox_cat', null, FALSE, array( 'getHtml' => function(){
				return "<span style='color:red;'>" . \IPS\Member::loggedIn()->language()->addToStack('fcontent_noVideobox') . "</span>";
			} ), function() {}, NULL, NULL, 'fcs_videobox_cat' ) );
		}		
		
		/* RSS */
		$form->add( new \IPS\Helpers\Form\Text( 'fcs_rssURL', $this->id ? $this->rssURL : FALSE, FALSE, array( 'placeholder' => 'http://www.example.com/rss' ), NULL, NULL, NULL, 'fcs_rssURL' ) );
		
		/* Specified contents */
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'fcs_featured', $this->featured ? explode( ',', $this->featured ) : array(), FALSE, array(
			'options' => array(
			    'pinned'      	=> 'pinned',
			    'unpinned'      => 'unpinned',
			    'featured'    	=> 'featured',			
			    'unfeatured'    => 'unfeatured',			
			)
		), NULL, NULL, NULL, 'fcs_featured' ) );
		
		$form->add( new \IPS\Helpers\Form\Text( 'fcs_tags', $this->id ? $this->tags : NULL, FALSE, array( 'maxLength' => 255 ), NULL, NULL, NULL, 'fcs_tags' ) );	
		
		$form->add( new \IPS\Helpers\Form\Number( 'fcs_date_range', isset( $this->date_range ) ? $this->date_range : 0, FALSE, array(
			'unlimited' => 0
		), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('fcs_date_range_suffix'), 'fcs_date_range' ) );						
	
		/* Sort contents */
		$form->addHeader( 'fcs_header_sorting' );		
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_sortkey', $this->id ? $this->sortkey : "id", FALSE, array(
			'options' => array(
					'id'					=> 'fcs_key_id',
					'start_date' 			=> 'fcs_key_start_date',
					'last_updated' 			=> 'fcs_key_last_updated',
					'title'					=> 'fcs_key_title',
					'views'					=> 'fcs_key_views',
					'rating'				=> 'fcs_key_topic_rating_total',
				) ), NULL, NULL, NULL, 'fcs_sortkey' ) );
	
		$form->add( new \IPS\Helpers\Form\Select( 'fcs_sortby', $this->id ? $this->sortby : "DESC", FALSE, array(
			'options' => array(
				'DESC' 	=> 'fcs_bydesc',
				'ASC' 	=> 'fcs_byasc',
			) ), NULL, NULL, NULL, 'fcs_sortby' ) );	
				
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_randomcontent', $this->id ? $this->randomcontent : 0, FALSE ) );


		$form->addHeader( 'fcs_tab_links' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'fcs_nolink', $this->id ? $this->nolink : 0, FALSE, array( 'togglesOff' => array( 'fcs_viewlinkin' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'fcs_viewlinkin', $this->id ? $this->viewlinkin : '1', FALSE, array( 'options' => array( '1' => 'fcs_viewin_current', '2' => 'fcs_viewin_page', '3' => 'fcs_viewin_popup' ) ), NULL, NULL, NULL, 'fcs_viewlinkin' ) );		
		
	}
	
	protected function getFieldsFromDB($id)
	{
		$data = array();
		
		$fclass = '\IPS\cms\Fields' . $id;
		$data[0] = '-----------';
		foreach( $fclass::roots() as $f )
		{
			$data[$f->id] = $f->_title;
		}
		
		return $data;
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
				
		/* Save CMS categories */
		if ( \IPS\Application::appIsEnabled( 'cms' ) && $values['fcs_cms_db'] )
		{		
			foreach( \IPS\cms\Databases::roots( NULL, NULL ) as $db )
			{
				if ( isset( $values[ 'fcs_cms_cat_' . $db->id ] ) && ( $db->id == $values['fcs_cms_db'] ) )
				{
					if ( $values[ 'fcs_cms_cat_' . $db->id ] == 0 )
					{
						$values['fcs_cms_cat'] = '*';
					}
					else 
					{
						$cmscats = array();
						foreach ( $values[ 'fcs_cms_cat_' . $db->id ] as $cat )
						{
							$cmscats[] = $cat->_id;
						}
						
						$values['fcs_cms_cat'] = ( implode( ',', $cmscats ) );
					}

					$values['fcs_cms_fieldIMG'] = $values[ 'fcs_cms_field_' . $db->id ];
				}
				
				unset( $values[ 'fcs_cms_cat_' . $db->id ] );
				unset( $values[ 'fcs_cms_field_' . $db->id ] );
			}
		}
		
		/* Save forums */
		if ( \IPS\Application::appIsEnabled( 'forums' ) && isset( $values['fcs_forums'] ) )
		{
			if ( $values['fcs_forums'] == 0 )
			{
				$values['fcs_forums'] = '*';
			}
			else 
			{
				$forums = array();
				foreach ( (array) $values['fcs_forums'] as $forum )
				{
					$forums[] = $forum->_id;
				}
				
				$values['fcs_forums'] = ( implode( ',', $forums ) );
			}		
		}		
		
		/* Save gallery */
		if ( \IPS\Application::appIsEnabled( 'gallery' ) && isset( $values['fcs_gallery_cat'] ) )
		{
			if ( $values['fcs_gallery_cat'] == 0 )
			{
				$values['fcs_gallery_cat'] = '*';
			}
			else 
			{
				$gcats = array();
				foreach ( (array) $values['fcs_gallery_cat'] as $gcat )
				{
					$gcats[] = $gcat->_id;
				}
				
				if ( \count($gcats) > 0 )
				{
					$values['fcs_gallery_cat'] = implode( ',', $gcats );
				}
			}
		}	
		
		/* Save downloads */
		if ( \IPS\Application::appIsEnabled( 'downloads' ) && isset( $values['fcs_downloads_cat'] ) )
		{
			if ( $values['fcs_downloads_cat'] == 0 )
			{
				$values['fcs_downloads_cat'] = '*';
			}
			else 
			{
				$dcats = array();
				foreach ( (array) $values['fcs_downloads_cat'] as $dcat )
				{
					$dcats[] = $dcat->_id;
				}
				if ( \count($dcats) > 0 )
				{				
					$values['fcs_downloads_cat'] = implode( ',', $dcats );
				}
			}				
		}	
		
		/* Save nexus */
		if ( \IPS\Application::appIsEnabled( 'nexus' ) && isset( $values['fcs_nexus_cat'] ) )
		{
			if ( $values['fcs_nexus_cat'] == 0 )
			{
				$values['fcs_nexus_cat'] = '*';
			}
			else 
			{
				$ncats = array();
				foreach ( (array) $values['fcs_nexus_cat'] as $ncat )
				{
					$ncats[] = $ncat->_id;
				}
				if ( \count($ncats) > 0 )
				{				
					$values['fcs_nexus_cat'] = implode( ',', $ncats );
				}
			}				
		}	
		
		/* Save videobox */
		if ( \IPS\Application::appIsEnabled( 'videobox' ) && isset( $values['fcs_videobox_cat'] ) )
		{
			if ( $values['fcs_videobox_cat'] == 0 )
			{
				$values['fcs_videobox_cat'] = '*';
			}
			else 
			{
				$vcats = array();
				foreach ( (array) $values['fcs_videobox_cat'] as $vcat )
				{
					$vcats[] = $vcat->_id;
				}
				if ( \count($vcats) > 0 )
				{				
					$values['fcs_videobox_cat'] = implode( ',', $vcats );
				}
			}				
		}			
		/* return */
		parent::saveForm( $values );
	}

	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		\IPS\Db::i()->delete( 'fcontent_sliders', array( 'fcs_id=?', $this->id ) );
			
		foreach( \IPS\Db::i()->select( '*', 'fcontent_contents', array( 'fcc_slider=?', $this->id ) ) as $slider )
		{
			if ( $slider['fcc_uploadimg'] )
			{
				try
				{
					\IPS\File::get( 'fcontent_Image', $slider['fcc_uploadimg'] )->delete();
				}
				catch( \Exception $e ){}
			}
		}		
		
		\IPS\Db::i()->delete( 'fcontent_contents', array( 'fcc_slider=?', $this->id ) );	
		
		return parent::delete();
	}
	
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'fcs_title';
		}

		if( $order == '_title' )
		{
			$order	= 'fcs_title';
		}

		return parent::search( $column, $query, $order, $where );
	}
	
	/**
	 * [ActiveRecord] Duplicate
	 *
	 * @return	void
	 */
	public function __clone()
	{
		if( $this->skipCloneDuplication === TRUE )
		{
			return;
		}

		$primaryKey = static::$databaseColumnId;
		$this->$primaryKey = NULL;
		
		$this->_new = TRUE;
		$this->save();
	}	
	
	/*-------------------------------------------------------------------------*/
	// get Contents in slider
	/*-------------------------------------------------------------------------*/		 
	protected static $_cacheContent = NULL;
	
	public function getContents()
	{
		if ( static::$_cacheContent[ $this->id ] !== NULL )
		{
			return static::$_cacheContent[ $this->id ];
		}	
		
		$contents = array();

		switch ( $this->method )
		{
			case 'cms':
				$contents = $this->fromCMS(); break;
				
			case 'forums':
				$contents = $this->fromForums(); break;
		
			case 'gallery':
				$contents = $this->fromGallery(); break;

			case 'downloads':
				$contents = $this->fromDownloads(); break;
				
			case 'nexus':
				$contents = $this->fromNexus(); break;
				
			case 'videobox':
				$contents = $this->fromVideobox(); break;
				
			case 'rss':
				$contents = $this->fromRSS(); break;
				
			case 'noauto':
			
			default:
				$contents = $this->fromCustom(); break;
		}
		
		static::$_cacheContent[ $this->id ] = $contents;
		
		return static::$_cacheContent[ $this->id ];
	}

	/*-------------------------------------------------------------------------*/
	// get Contents from Pages (CMS)
	/*-------------------------------------------------------------------------*/		
	protected function fromCMS()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "primary_id_field"; break;
				
			case 'start_date':
				$sort = "record_publish_date"; break;
				
			case 'last_updated':
				$sort = "record_last_comment"; break;
											
			case 'title':
				$sort = "record_dynamic_furl"; break;
				
			case 'views':
				$sort = "record_views"; break;
			
			case 'rating':
				$sort = "rating_real"; break;
				
			default:
				$sort = "primary_id_field"; break;
		}		
		
		if ( $this->cms_fieldIMG > 0 )
		{
			$fieldsClass = '\IPS\cms\Fields' . $this->cms_db;
			$fields = $fieldsClass::load( $this->cms_fieldIMG );
		}

		$recordClass = 'IPS\cms\Records' . $this->cms_db;
		$dbName = 'cms_custom_database_' . $this->cms_db;
		$where[] = array($dbName . ".record_approved=1");
		
		if ( $this->cms_cat !== '*' )
		{
			$where[] = array( $dbName . ".record_approved=1 && " . $dbName . ".category_id IN (". $this->cms_cat .")" );
		}
		
		if ( !empty( $this->featured ) )
		{
			$status = explode(",", $this->featured );
			if ( \in_array( 'pinned', $status ) )
			{
				$state[] = $dbName . '.record_pinned=1';
			}
			if ( \in_array( 'unpinned', $status ) )
			{
				$state[] = $dbName . '.record_pinned=0';
			}			
			if ( \in_array( 'featured', $status ) )
			{
				$state[] = $dbName . '.record_featured=1';
			}
			if ( \in_array( 'unfeatured', $status ) )
			{
				$state[] = $dbName . '.record_featured=0';
			}			
			$where[] = array( "(" . implode(" OR ", $state) . ")" );
		}
		
		if ( $this->tags )
		{
			$query = \IPS\Content\Search\Query::init();
			$filter = \IPS\Content\Search\ContentFilter::init( "IPS\cms\Records" . $this->cms_db );
			$query->filterByContent( array( $filter ) );
			$query->setOrder( \IPS\Content\Search\Query::ORDER_NEWEST_CREATED );
			$results = $query->search( null, explode(",", $this->tags), 17 );
			
			if ( $results && \count($results) > 0 )
			{
				foreach( $results as $result )
				{
					$rows[] = $this->convertObjToArr( $result )['index_item_id'];
				}
				$where[] = array( "primary_id_field IN (". implode(",", $rows) .")" );
			}
		}
		
		if ( isset( $this->date_range ) and intval( $this->date_range ) > 0 )
		{
			$where[] = array( 'record_publish_date>=?', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->date_range . 'D' ) )->getTimestamp() );
		}
		
		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;
		$records = $recordClass::getItemsWithPermission( $where, $sort, $this->total_items );

		//$records = \IPS\Db::i()->select( '*', 'cms_custom_database_' . $this->cms_db, $where, $sort, array( 0, $this->total_items ) );
		foreach( $records as $k => $row )
		{		
			try
			{		
				//$row = $recordClass::constructFromData( $r );			

				if ( $fields )
				{
					$fieldIMG = "field_" . $this->cms_fieldIMG;
					
					if ($fields->type == 'Upload')
					{
						if ( $fields->is_multiple == 1 )
						{
							$imgs_ = explode(",", $row->$fieldIMG);
							$imgFromField = $imgs_[0];
						}
						else
						{
							$imgFromField = $row->$fieldIMG;
						}
						
						$image = $imgFromField ? (string) \IPS\File::get( 'cms_Records', $imgFromField )->url : null;
					}
					else
					{
						$image = $this->getImageInPost( $row->$fieldIMG );
					}
				}
				else
				{
					$image = $row->record_image ? (string) \IPS\File::get( 'cms_Records', $row->record_image )->url : $this->getImageInPost($row->content());
				}

				$data[$k] = array( 	'image'  	=> $this->thumbnail($image),
									'title'  	=> $row->_title,
									'url'	 	=> (string) $row->url( 'getNewComment' ),
									'author' 	=> $row->author(),
									'tag'		=> \count($row->tags()) ? $row->tags()[0] : null,
									'rating'	=> $row->rating,
									'views'		=> $row->views,
									'comments'	=> $row->comments,
									'content'	=> $this->cutText($row->content(),200),
									'date'		=> $row->record_saved,
									'id'		=> $row->primary_id_field,
								);
			}
			catch( \Exception $ex )
			{
			}
		}
		return $data;
	}
	
	/*-------------------------------------------------------------------------*/
	// get Contents from Forums
	/*-------------------------------------------------------------------------*/		
	protected function fromForums()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "tid"; break;
				
			case 'start_date':
				$sort = "start_date"; break;
				
			case 'last_updated':
				$sort = "last_real_post"; break;
											
			case 'title':
				$sort = "title"; break;
				
			case 'views':
				$sort = "views"; break;
			
			case 'rating':
				$sort = "topic_rating_total"; break;
				
			default:
				$sort = "tid"; break;
		}
		
		$where[] = array( "approved=1 AND topic_archive_status!=1" );
		
		if ( $this->forums !== '*' )
		{
			$where[] = array( "forum_id IN (" . $this->forums . ")" );
		}

		if ( !empty( $this->featured ) )
		{
			$status = explode(",", $this->featured );
			if ( \in_array( 'pinned', $status ) )
			{
				$state[] = 'pinned=1';
			}
			if ( \in_array( 'unpinned', $status ) )
			{
				$state[] = 'pinned=0';
			}			
			if ( \in_array( 'featured', $status ) )
			{
				$state[] = 'featured=1';
			}
			if ( \in_array( 'unfeatured', $status ) )
			{
				$state[] = 'featured=0';
			}			
			$where[] = array( "(" . implode(" OR ", $state) . ")" );
		}	

		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;
		
		$tlimit = $this->skiptopicnoimg == 1 ? $this->total_items * 10 : $this->total_items;
		
		$num = 0;
		
		foreach ( \IPS\forums\Topic::getItemsWithPermission( $where, $sort, $tlimit ) as $k => $topic )
		{
			if ( $this->fromtthumb == 1 && method_exists($topic,'tthumb_show') )
			{
				$thumb = $topic->tthumb_show(1);
				$topicThumbnail_noimg = \IPS\Settings::i()->bim_tthumb_defaultimg ? (string) \IPS\File::get( 'core_Theme', \IPS\Settings::i()->bim_tthumb_defaultimg )->url : (string) \IPS\Theme::i()->resource( 'plugins/noThumb.png', 'core', 'global' );
				if ( $thumb == $topicThumbnail_noimg )
				{
					$thumb = null;
				}
			}
			else
			{
				$postcontent = $this->getlastimage != 1 ? $topic->content() : $topic->comments( 1, 0, 'date', 'DESC' )->post;
				$thumb = $this->getImageInPost( $postcontent );
			}

			if ( $this->skiptopicnoimg == 1 )
			{
				if ( $thumb == null )
				{
					continue;
				}

				$num++;
				
				if ( $num >= $this->total_items )
				{
					break;
				}
			}

			$data[$k] = array( 	'image'  	=> $this->thumbnail($thumb),
								'title'  	=> $topic->title,
								'url'	 	=> $re[0]['id2'] ? (string) $topic->url()->setQueryString( array( 'do' => 'findComment', 'comment' => $re[0]['id2'] ) ) : (string) $topic->url(),
								'author' 	=> $topic->author(),
								'tag'		=> \count($topic->tags()) ? $topic->tags()[0] : null,	
								'rating'	=> $topic->rating,										
								'views'		=> $topic->views,
								'comments'	=> $topic->posts - 1,
								'content'	=> $this->cutText($topic->content(),200),
								'date'		=> $topic->start_date,
								'id'		=> $topic->tid,
			);
		}			

		return $data;
	}

	/*-------------------------------------------------------------------------*/
	// get Contents from Gallery
	/*-------------------------------------------------------------------------*/		
	protected function fromGallery()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "image_id"; break;
				
			case 'start_date':
				$sort = "image_date"; break;
				
			case 'last_updated':
				$sort = "image_updated"; break;
											
			case 'title':
				$sort = "image_caption"; break;
				
			case 'views':
				$sort = "image_views"; break;
			
			case 'rating':
				$sort = "image_ratings_total"; break;
				
			default:
				$sort = "image_id"; break;
		}		
		
		$where[] = array("image_approved=1");
		
		if ( $this->gallery_cat !== '*' )
		{
			$where[] = array("image_category_id IN (". $this->gallery_cat .")");
		}
		
		if ( !empty( $this->featured ) )
		{
			$status = explode(",", $this->featured );
			if ( \in_array( 'pinned', $status ) )
			{
				$state[] = 'image_pinned=1';
			}
			if ( \in_array( 'unpinned', $status ) )
			{
				$state[] = 'image_pinned=0';
			}			
			if ( \in_array( 'featured', $status ) )
			{
				$state[] = 'image_feature_flag=1';
			}
			if ( \in_array( 'unfeatured', $status ) )
			{
				$state[] = 'image_feature_flag=0';
			}			
			$where[] = array( "(" . implode(" OR ", $state) . ")" );
		}
		
		if ( isset( $this->tags ) )
		{
			$query = \IPS\Content\Search\Query::init();
			$filter = \IPS\Content\Search\ContentFilter::init( "IPS\gallery\Image" );
			$query->filterByContent( array( $filter ) );
			$query->setOrder( \IPS\Content\Search\Query::ORDER_NEWEST_CREATED );
			$results = $query->search( null, explode(",", $this->tags), 17 );
			
			if ( $results && \count($results) > 0 )
			{
				foreach( $results as $result )
				{
					$rows[] = $this->convertObjToArr( $result )['index_item_id'];
				}
				$where[] = array( "image_id IN (". implode(",", $rows) .")" );
			}
		}
		
		if ( isset( $this->date_range ) and intval( $this->date_range ) > 0 )
		{
			$where[] = array( 'image_date>=?', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->date_range . 'D' ) )->getTimestamp() );
		}	
		
		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;
		
		foreach ( \IPS\gallery\Image::getItemsWithPermission( $where, $sort, $this->total_items ) as $k => $image )
		{
			try
			{			
				$file = $image->medium_file_name? $image->medium_file_name : $image->original_file_name;
				$thumb = (string) \IPS\File::get( 'gallery_Images', $file )->url;
			}
			catch( \Exception $e ) {}				
			$data[$k] = array( 	'image'  	=> $this->thumbnail($thumb),
								'title'  	=> $image->caption,
								'url'	 	=> (string) $image->url(),
								'author' 	=> $image->author(),
								'tag'		=> \count($image->tags()) ? $image->tags()[0] : null,	
								'rating'	=> $image->rating,										
								'views'		=> $image->views,
								'comments'	=> $image->comments,
								'content'	=> $this->cutText($image->content(),200),
								'date'		=> $image->date,
								'id'		=> $image->id,
							);
		}
		return $data;
	}
	
	
	/*-------------------------------------------------------------------------*/
	// get Contents from Downloads
	/*-------------------------------------------------------------------------*/	
	protected function fromDownloads()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "file_id"; break;
				
			case 'start_date':
				$sort = "file_submitted"; break;
				
			case 'last_updated':
				$sort = "file_updated"; break;
											
			case 'title':
				$sort = "file_name"; break;
				
			case 'views':
				$sort = "file_views"; break;
			
			case 'rating':
				$sort = "file_rating"; break;
				
			default:
				$sort = "file_id"; break;
		}		
		
		$where[] = array( "file_open=1" );
		
		if ( $this->downloads_cat !== '*' )
		{
			$where[] = array( "file_cat IN (". $this->downloads_cat .")" );
		}
	
		if ( !empty( $this->featured ) )
		{
			$status = explode(",", $this->featured );
			if ( \in_array( 'pinned', $status ) )
			{
				$state[] = 'file_pinned=1';
			}
			if ( \in_array( 'unpinned', $status ) )
			{
				$state[] = 'file_pinned=0';
			}			
			if ( \in_array( 'featured', $status ) )
			{
				$state[] = 'file_featured=1';
			}
			if ( \in_array( 'unfeatured', $status ) )
			{
				$state[] = 'file_featured=0';
			}			
			$where[] = array( "(" . implode(" OR ", $state) . ")" );
		}	
		
		if ( isset( $this->tags ) )
		{
			$query = \IPS\Content\Search\Query::init();
			$filter = \IPS\Content\Search\ContentFilter::init( "IPS\downloads\File" );
			$query->filterByContent( array( $filter ) );
			$query->setOrder( \IPS\Content\Search\Query::ORDER_NEWEST_CREATED );
			$results = $query->search( null, explode(",", $this->tags), 17 );
			
			if ( $results && \count($results) > 0 )
			{
				foreach( $results as $result )
				{
					$rows[] = $this->convertObjToArr( $result )['index_item_id'];
				}
				$where[] = array( "file_id IN (". implode(",", $rows) .")" );
			}
		}
		
		if ( isset( $this->date_range ) and intval( $this->date_range ) > 0 )
		{
			$where[] = array( 'file_submitted>=?', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->date_range . 'D' ) )->getTimestamp() );
		}	

		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;
		
		foreach( \IPS\downloads\File::getItemsWithPermission( $where, $sort, $this->total_items ) as $k => $file )
		{
			try
			{
				$image = (string) \IPS\File::get( 'downloads_Screenshots', $file->primary_screenshot )->url;
			}
			catch( \Exception $e ) {}
			
			$data[$k] = array( 	'image'  	=> $this->thumbnail($image),
								'title'  	=> $file->name,
								'url'	 	=> (string) $file->url(),
								'author' 	=> $file->author(),
								'tag'		=> \count($file->tags()) ? $file->tags()[0] : null,	
								'rating'	=> $file->rating,										
								'views'		=> $file->views,
								'comments'	=> $file->comments,
								'content'	=> $this->cutText($file->content(),200),
								'date'		=> $file->submitted,
								'id'		=> $file->id,
							);
		}
		
		return $data;		
	}
	
	/*-------------------------------------------------------------------------*/
	// get Contents from Nexus
	/*-------------------------------------------------------------------------*/	
	protected function fromNexus()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "p_id"; break;
				
			case 'start_date':
				$sort = "p_date_added"; break;
				
			case 'last_updated':
				$sort = "p_date_updated"; break;
											
			case 'title':
				$sort = "p_seo_name"; break;
				
			case 'views':
				$sort = "p_reviews"; break;
			
			case 'rating':
				$sort = "p_rating"; break;
				
			default:
				$sort = "p_id"; break;
		}		
		
		if ( $this->nexus_cat !== '*' )
		{
			$where[] = array( "p_group IN (". $this->nexus_cat .")" );
		}
		
		if ( isset( $this->date_range ) and intval( $this->date_range ) > 0 )
		{
			$where[] = array( 'p_date_added>=?', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->date_range . 'D' ) )->getTimestamp() );
		}	
		
		$where[] = array( 'p_store=1' );
		
		$where[] = array( "( p_member_groups='*' OR " . \IPS\Db::i()->findInSet( 'p_member_groups', \IPS\Member::loggedIn()->groups ) . ' )' );
		
		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;

		$select = \IPS\Db::i()->select( '*', 'nexus_packages', $where, $sort, array( 0, $this->total_items ) );

		if ( $select && \count( $select ) > 0 )
		{
			$packages = new \IPS\Patterns\ActiveRecordIterator( $select, 'IPS\nexus\Package' );
			
			foreach( $packages as $k => $item )
			{
				
				$data[$k] = array( 'image'  		=> $item->image,
									'title'  		=> $item->_title,
									'url'			=> (string) $item->url(),
									'author' 		=> NULL,
									'tags' 			=> NULL,
									'rating'		=> $item->rating,
									'content'		=> NULL,
									'date'			=> $item->date_updated,
									'id'			=> $item->date_added,
											
				);							
			}
		}

		return $data;		
	}
	
	/*-------------------------------------------------------------------------*/
	// get Contents from Videobox
	/*-------------------------------------------------------------------------*/	
	protected function fromVideobox()
	{
		switch ( $this->sortkey )
		{
			case 'id':
				$sort = "v_id"; break;
				
			case 'start_date':
				$sort = "v_started"; break;
				
			case 'last_updated':
				$sort = "v_updated"; break;
											
			case 'title':
				$sort = "v_title"; break;
				
			case 'views':
				$sort = "v_views"; break;
			
			case 'rating':
				$sort = "v_rating_average"; break;
				
			default:
				$sort = "v_id"; break;
		}		
		
		$where[] = array( "v_open=1" );
		
		if ( $this->videobox_cat !== '*' )
		{
			$where[] = array( "v_cat IN (". $this->videobox_cat .")" );
		}
	
		if ( !empty( $this->featured ) )
		{
			$status = explode(",", $this->featured );
			if ( \in_array( 'pinned', $status ) )
			{
				$state[] = 'v_pinned=1';
			}
			if ( \in_array( 'unpinned', $status ) )
			{
				$state[] = 'v_pinned=0';
			}			
			if ( \in_array( 'featured', $status ) )
			{
				$state[] = 'v_featured=1';
			}
			if ( \in_array( 'unfeatured', $status ) )
			{
				$state[] = 'v_featured=0';
			}			
			$where[] = array( "(" . implode(" OR ", $state) . ")" );
		}	
		
		if ( isset( $this->tags ) )
		{
			$query = \IPS\Content\Search\Query::init();
			$filter = \IPS\Content\Search\ContentFilter::init( "IPS\videobox\Video" );
			$query->filterByContent( array( $filter ) );
			$query->setOrder( \IPS\Content\Search\Query::ORDER_NEWEST_CREATED );
			$results = $query->search( null, explode(",", $this->tags), 17 );
			
			if ( $results && \count($results) > 0 )
			{
				foreach( $results as $result )
				{
					$rows[] = $this->convertObjToArr( $result )['index_item_id'];
				}
				$where[] = array( "v_id IN (". implode(",", $rows) .")" );
			}
		}
		
		if ( isset( $this->date_range ) and intval( $this->date_range ) > 0 )
		{
			$where[] = array( 'v_started>=?', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $this->date_range . 'D' ) )->getTimestamp() );
		}	
		
		$sort = $this->randomcontent == 1 ? "RAND()" : $sort . " " . $this->sortby;
		
		foreach( \IPS\videobox\Video::getItemsWithPermission( $where, $sort, $this->total_items ) as $k => $video )
		{
			$data[$k] = array( 	'image'  	=> $this->thumbnail($video->thumbnail('medium')),
								'title'  	=> $video->title,
								'url'	 	=> (string) $video->url(),
								'author' 	=> $video->author(),
								'tag'		=> \count($video->tags()) ? $video->tags()[0] : null,	
								'rating'	=> $video->rating_average,										
								'views'		=> $video->views,
								'comments'	=> $video->comments,
								'content'	=> $this->cutText($video->content(),200),
								'date'		=> $video->started,
								'id'		=> $video->id,
							);
		}

		return $data;		
	}	
	
	/*-------------------------------------------------------------------------*/
	// get Contents from RSS
	/*-------------------------------------------------------------------------*/		
	protected function fromRSS()
	{
		$data = array();
		
		if ( !$this->rssURL )
		{
			return '';
		}
		try
		{
			$request = \IPS\Http\Url::external( $this->rssURL )->request()->get();
			$content = $request->content;
			$request = $request->decodeXml();
		}
		catch ( \IPS\Http\Url\Exception $e ){}
		catch ( \IPS\Http\Exception $e ) {}
		catch( \IPS\Http\Request\Exception $e ) {}
		catch( \RuntimeException $e ) {}

		if ( mb_stripos( $content, "<content:encoded>" ) !== false )
		{
			$rss = new \DOMDocument();
			$rss->loadXML($content);
			$i = 0;
			foreach ($rss->getElementsByTagName('item') as $node) 
			{
				if ( $this->skiptopicnoimg == 1 || !$node->getElementsByTagName('title')->item(0)->nodeValue)
				{
					if ( $this->getImageInPost($node->getElementsByTagName('encoded')->item(0)->nodeValue) == $this->noImageURL() )
					{
						continue;
					}
				}
				$image = $this->getImageInPost($node->getElementsByTagName('encoded')->item(0)->nodeValue);
				$data[] = array (
						'image'		=> $this->thumbnail($image),
						'title' 	=> $node->getElementsByTagName('title')->item(0)->nodeValue,
						'url' 		=> $node->getElementsByTagName('link')->item(0)->nodeValue
				);
			}
		}
		else
		{
			if ( \count( $request ) > 0 )
			{
				if ( $request instanceof \IPS\Xml\Atom )
				{
					$i = 0;

					foreach ( $request as $entry )
					{
						if ( $this->skiptopicnoimg == 1 || !$entry->title)
						{
							if ( $this->getImageInPost( (string) $entry->content ) == $this->noImageURL() )
							{
								continue;
							}
						}
						
						$data[] = $entry;
						
						if ( $i++ == $this->total_items )
						{
							break;
						}				
					}
					
					if ( \count($data) > 0 )
					{
						foreach ( $data as $k => $entry )
						{
							$image = $this->getImageInPost( (string) $entry->content );
							$data[$k] = array( 	'image'  => $this->thumbnail($image),
												'title'  => (string) $entry->title,
												'url'	 => $entry->link['href'] ? $entry->link['href'] : (string) $entry->link,
											);		
						}
					}
				}
				else
				{
					$i = 0;
					foreach ( $request->channel->item as $entry )
					{
						if ( $this->skiptopicnoimg == 1 || !$entry->title)
						{
							if ( $this->rssIMG($entry) == $this->noImageURL() )
							{
								continue;
							}
						}
						
						$data[] = $entry;
						
						if ( $i++ == $this->total_items )
						{
							break;
						}				
					}
					
					if ( \count($data) > 0 )
					{
						foreach ( $data as $k => $entry )
						{
							$image = $this->rssIMG($entry);
							$data[$k] = array( 	'image'  => $this->thumbnail($image), 
												'title'  => (string) $entry->title,
												'url'	 => (string) $entry->link,
											);		
						}
					}
				}

			}
		}
		
		if ( $this->randomcontent == 1 && \count($data) > 0)
		{
			shuffle($data);
		}		
		
		return $data;
	}
	
	protected function rssIMG($entry)
	{
		if ( $thumbAttr = $entry->children('media', true) )
		{
			$thumbAttr = $thumbAttr->content->attributes();
			return $thumbAttr['url'];
		}
		if ( isset($entry->enclosure) )
		{
			return $entry->enclosure->attributes()->url;
		}
		else return $this->getImageInPost( (string) $entry->description );
	}
	
	/*-------------------------------------------------------------------------*/
	// get Contents Manually
	/*-------------------------------------------------------------------------*/		
	protected function fromCustom()
	{
		$sort = $this->randomcontent == 1 ? "RAND()" : "fcc_position ASC";
		
		foreach ( \IPS\Db::i()->select( '*', 'fcontent_contents', "fcc_slider=" . $this->id, $sort, array( 0, $this->total_items ) ) as $k => $row )
		{
			$img = $row['fcc_imageFrom'] == 'upload' ? (string) \IPS\File::get( 'fcontent_Image', $row['fcc_uploadimg'] )->url : $row['fcc_image'];
			
			$data[$k] = array( 	'image'  		=> $this->thumbnail($img),
								'title'  		=> strip_tags( $row['fcc_title'] ),
								'url'	 		=> $row['fcc_url'],
								'id'	 		=> $row['fcc_id'],
								'moderators'	=> $row['fcc_moderators'],
								'newtab'		=> $row['fcc_newtab'],
							);
		}
		return $data;	
	}
	
	/*-------------------------------------------------------------------------*/
	// Get first image in first post
	/*-------------------------------------------------------------------------*/		
	public function getImageInPost($content=null)
	{
		$thumbs = array();
		try
		{			
			$loadPost = !$content ? \IPS\forums\Topic\Post::load( $this->topic_firstpost )->post : $content;
			
			$source = new \IPS\Xml\DOMDocument( '1.0', 'UTF-8' );
			$source->loadHTML( \IPS\Xml\DOMDocument::wrapHtml( $loadPost ) );
			$contentImages = $source->getElementsByTagName( 'img' );
			foreach( $contentImages as $el )
			{
				if ( \IPS\Settings::i()->lazy_load_enabled )
				{
					$datasrc = $this->validURL( $el->hasAttribute('data-src') ? $el->getAttribute( 'data-src' ) : $el->getAttribute( 'src' ) );

					if ( mb_strpos( $datasrc, 'fileStore.core_Emoticons' ) === false )
					{
						$thumbs[] = $this->getRealIMG( $datasrc );
					}					
				}
				else
				{
					if ( $el->hasAttribute('src') )
					{
						$src = $this->validURL( $el->getAttribute( 'src' ) );
						if ( mb_strpos( $src, 'fileStore.core_Emoticons' ) === false )
						{
							$thumbs[] = $this->getRealIMG( $src );
						}
					}
				}
			}
			
			$contentIframes = $source->getElementsByTagName( 'iframe' );
			
			foreach( $contentIframes as $el )
			{
				if ( \IPS\Settings::i()->lazy_load_enabled )
				{
					$thumbs[] = $this->videoIMG( $el->hasAttribute('data-embed-src') ? $el->getAttribute( 'data-embed-src' ) : $el->getAttribute( 'src' ) );					
				}
				else
				{
					if ( \mb_strpos( $el->getAttribute( 'src' ), 'core/interface/index.html') !== FALSE )
					{
						$iframeSrc = $el->getAttribute( 'data-embed-src' );
					}
					else
					{
						$iframeSrc = $el->getAttribute( 'src' );
					}
					$thumbs[] = $this->videoIMG( $iframeSrc );
				}
			}

			preg_match_all( '/\[photo=(.+?)\](.+?)\[\/photo\]/i', $loadPost, $match );
			for ( $i = 0 ; $i < \count($match[0]) ; $i++ )
			{
				$thumbs[] = $this->getRealIMG( strip_tags($match[1][$i]) );
			}
			
			preg_match_all( '/\[photo=(.+?)\]\[\/photo\]/i', $loadPost, $match );
			for ( $i1 = 0 ; $i1 < \count($match[0]) ; $i1++ )
			{
				$thumbs[] = $this->getRealIMG( strip_tags($match[1][$i1]) );
			}
			
			preg_match_all( '/\[img\](.+?)\[\/img\]/i', $loadPost, $match );
			for ( $i2 = 0 ; $i2 < \count($match[0]) ; $i2++ )
			{
				$thumbs[] = $this->getRealIMG( strip_tags($match[1][$i2]) );
			}			
		}
		catch ( \Exception $e ) {}	
		
		return $thumbs[0];
	}
	
	protected function isIgnoreUrl( $url )
	{
		$iglist = array();
		
		$iglist = explode( "\n", str_replace( "\r", "", \IPS\Settings::i()->bim_fc_ignoreurls ) );
		$iglist = array_filter($iglist);
		
		$return = false;
		
		if ( \count($iglist) > 0 )
		{
			foreach( $iglist as $u )
			{
				if (mb_strpos($url, $u) !== FALSE)
				{
					$return = true;
					break;
				}
			}
		}
		return $return;
	}
	
	protected function getRealIMG($txt)
	{
		if ( mb_strpos( $txt, 'fileStore.core_Attachment' ) !== false )
		{
			$isAtt = true;
		}
		$txt = str_replace("fileStore.core_Emoticons", "", $txt);
		$txt = str_replace("fileStore.core_Attachment", "", $txt);
		$txt = str_replace("<>", "", $txt);			
		$txt = str_replace("%3C%3E", "", $txt);
		$txt = str_replace( '<___base_url___>', rtrim( \IPS\Settings::i()->base_url, '/' ), $txt );
		
		if ( $isAtt ) 
		{
			$txt = (string) \IPS\File::get( 'core_Attachment', $txt )->url;
		}

		// Fix double slashes	
		$parts = explode('//', $txt, 2);
		$parts[1] = rtrim(preg_replace('@/+@', '/', $parts[1]), '/');
		$txt = implode('//', $parts);
		unset($parts);

		// Remove double slashes at the end
		if (mb_strpos($txt, "fileStore.") !== false || mb_strpos($txt, "___base_url___") !== false)
		{
			$txt = rtrim($txt,"/");
		}		
		
		// Done
		return $txt;
	}
	
	/*-------------------------------------------------------------------------*/
	// valid URL
	/*-------------------------------------------------------------------------*/
	public function validURL( $url )
	{
		if ( !preg_match("~^(?:f|ht)tps?:~i", $url) && \mb_strpos( $url, "___base_url___" ) === FALSE && \mb_strpos( $url, "fileStore.core_Attachment" ) === FALSE )
		{
			if ( mb_strpos( $url, "//" ) === 0 )
			{
				$url = ( mb_substr( \IPS\Settings::i()->base_url, 0, 5 ) === 'https' ) ? 'https:' : 'http:' . $url;
			}
			else
			{
				$url = rtrim( \IPS\Settings::i()->base_url, '/' ) . '/' . ltrim( $url, '/' );
			}
		}
		return $url;
	}	
	
	/*-------------------------------------------------------------------------*/
	// Show video thumbnail ( support youtube & vimeo )
	/*-------------------------------------------------------------------------*/	
	protected function videoIMG( $txt )
	{
		if ( preg_match( '#^(?:https?://)?(?:www\.)?(?:m\.)?(?:youtu\.be/|youtube\.com(?:/embed/|/v/|/watch\?v=|/watch\?.+&v=))([\w-]{11})(?:.+)?$#x', $txt, $match ) )
		{
			$thumb = "https://img.youtube.com/vi/{$match[1]}/hqdefault.jpg";
		}
		elseif ( preg_match('/https?:\/\/(.+)?(wistia.com|wi.st|net)\/(medias|embed)\/(.*)\/(.*)/', $txt, $match ) )
		{
			$wID = $match[5];
			$url = "https://fast.wistia.com/oembed?url=http://home.wistia.com/medias/" . $wID;
			$wData = \IPS\Http\Url::external( $url )->request()->get()->decodeJson();
			$img = explode("?", $wData['thumbnail_url']);
			$width = $this->img_w ? $this->img_w : 310;
			$height = $this->img_h ? $this->img_h : 210;
			$thumb = $img[0] . "?image_crop_resized=" . $width . "x" . $height;
		}
		elseif ( preg_match( "/dailymotion.com\/(.*?)/", $txt, $match ) )
		{
			$id = strtok(basename($txt), '_');
			try
			{
				$request = \IPS\Http\Url::external( "https://api.dailymotion.com/video/" . $id . "?fields=thumbnail_480_url" )->request()->get();
				$output = json_decode($request);
				return $output->thumbnail_480_url;
			}
			catch ( \Exception $e ) {}	
		}
		elseif ( preg_match( "/vimeo.com\/([1-9.-_]+)/", $txt, $match ) || preg_match( "/vimeo.com\/video\/([1-9.-_]+)/", $txt, $match ) )
		{
			$vurls = explode( "/", $txt );
			$vmid = $vurls[ \count( $vurls ) - 1 ];
			try
			{
				$request = \IPS\Http\Url::external( "https://vimeo.com/api/oembed.json?url=https://player.vimeo.com/video/" . $vmid )->request()->get();
				$output = json_decode( $request );
				return $output->thumbnail_url;					
			}
			catch ( \Exception $e ) {}
		}		
		return $thumb;
	}	

	/*-------------------------------------------------------------------------*/
	// truncateText
	/*-------------------------------------------------------------------------*/
	public function truncateText( $text, $chars=40 )
	{	
		if ( mb_strlen($text) > $this->truncate )
		{
			$text = $text." ";
			$text = mb_substr($text, 0, $chars);
			$text = mb_substr($text, 0, mb_strrpos($text,' '));
			$text = $text."...";			
		}
		return $text;
	}	

	protected function cutText( $text, $chars )
	{	
		$text = strip_tags($text);
		if ( mb_strlen($text) > $chars )
		{
			$text = $text." ";
			$text = mb_substr($text, 0, $chars);
			$text = mb_substr($text, 0, mb_strrpos($text,' '));
			$text = $text."...";			
		}
		return $text;
	}	
	
	
	/*-------------------------------------------------------------------------*/
	// NoImageURL
	/*-------------------------------------------------------------------------*/
	protected function noImageURL()
	{
		return $this->noimg ? (string) \IPS\File::get( 'core_Theme', $this->noimg )->url : (string) \IPS\Theme::i()->resource( 'noThumb.png', 'fcontent' );
	}
		
	/*-------------------------------------------------------------------------*/
	// Cache Image
	/*-------------------------------------------------------------------------*/
	protected $_cache = NULL;
	
	protected function thumbnail( $img )
	{
		# No img
		if ( !$img || $img == $this->noImageURL() )
		{
			return $this->noImageURL();
		}
		
		if ( \IPS\Settings::i()->bim_fc_cache != 1 )
		{
			return $img;
		}
		
		// Do not upload thumbnails from video's services
		$parseUrl = parse_url( $img );
		$host = str_replace( 'www.', '', trim( $parseUrl['host'] ? $parseUrl['host'] : array_shift( explode('/', $parseUrl['path'], 2) ) ) ); 
		if ( \in_array( $host, array( 'i.ytimg.com', 'img.youtube.com', 'i.vimeocdn.com', 's1.dmcdn.net', 'static-cdn.jtvnw.net' ) ) )
		{
			return $img;
		}
		
		# Get cache
		if ( $this->_cache[ $this->id ] === NULL )
		{
			$this->_cache[ $this->id ] = array();
			
			foreach( \IPS\Db::i()->select( '*', 'fcontent_cache', array( 'slider=?', $this->id ) ) as $row )
			{
				try
				{
					$file = \IPS\File::get( 'fcontent_Cache', $row['cache'] );
	
					if ( !$file->filesize() )
					{	
						\IPS\Db::i()->delete( 'fcontent_cache', array( 'id=?', $row['id'] ) );					
						continue;
					}
					
					$this->_cache[ $this->id ][ md5( $row['image'] ) ] = $file;
				}
				catch ( \Exception $e )
				{
					$this->_cache[ $this->id ][ md5( $row['image'] ) ] = $img;
				}				
			}
		}
		
		# Create cache
		$key = md5( $img );
		
		if ( ! $this->_cache[ $this->id ][ $key ] )
		{
			$rawIMG = $this->getImageRawData( $img );

			if ( $rawIMG && @imagecreatefromstring($rawIMG) !== false )
			{
				try
				{				
					$temp = \IPS\File::create( 'fcontent_Cache', "fcontent." . mb_substr( mb_strrchr( $img, '.' ),1 ), $rawIMG );
					
					$thumb = $temp->thumbnail( 'fcontent_Cache', $this->img_w * 2, $this->img_h * 2 );
					$temp->delete();
					
					\IPS\Db::i()->insert( 'fcontent_cache', array(
						'slider'	=> $this->id,
						'cache'		=> (string) $thumb,
						'image'		=> $img
					) );
					
					$this->_cache[ $this->id ][ $key ] = $thumb;				
				}
				catch ( \Exception $e )
				{
					$this->_cache[ $this->id ][ $key ] = $img;
				}
			}
			else
			{
				$this->_cache[ $this->id ][ $key ] = $img;
			}
		}

		# Done
		return \is_object( $this->_cache[ $this->id ][ $key ] ) ? (string) $this->_cache[ $this->id ][ $key ]->url : $this->_cache[ $this->id ][ $key ];
	}

	protected function getImageRawData($image_url) 
	{
		if ( \function_exists('curl_init') ) 
		{
			$ch = \curl_init();
			$timeout = 5;
			curl_setopt($ch, CURLOPT_URL, $image_url);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);	
			$content = curl_exec($ch);
			$statusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			curl_close($ch);
			
			if ( $statusCode != 200 )
			{
				return null;
			}
		}
		elseif ( ini_get('allow_url_fopen') ) 
		{
			$content = @file_get_contents( $image_url );
		}
		
		return $content;
	}	
	
	protected function convertObjToArr($result)
	{
		$result = (array) $result;
		foreach($result as $r)
		{
			$data[] = (array) $r;
		}
		$data = \call_user_func_array('array_merge', $data);
		return $data;
	}	
	
	
	
	
	
	
	
}