<?php
/**
 * @brief		fcWidget Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	fcontent
 * @since		08 Aug 2019
 */

namespace IPS\fcontent\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * fcWidget Widget
 */
class _fcWidget extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'fcWidget';
	
	/**
	 * @brief	App
	 */
	public $app = 'fcontent';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * @brief	Cache Expiration
	 * @note	We allow this cache to be valid for 5 minutes
	 */
	//public $cacheExpiration = 300;
	
	
	public static $loadedJSandCSS = false;

	/**
	 * Constructor
	 *
	 * @param	String				$uniqueKey				Unique key for this specific instance
	 * @param	array				$configuration			Widget custom configuration
	 * @param	null|string|array	$access					Array/JSON string of executable apps (core=sidebar only, content=IP.Content only, etc)
	 * @param	null|string			$orientation			Orientation (top, bottom, right, left)
	 * @return	void
	 */	
	public function __construct( $uniqueKey, array $configuration, $access=null, $orientation=null )
	{
		parent::__construct( $uniqueKey, $configuration, $access, $orientation );
		
		if ( ! self::$loadedJSandCSS )
		{
			self::$loadedJSandCSS = true;
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_slider.js', 'fcontent', 'front' ) );
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'slider.css', 'fcontent', 'front' ) );
		}	
		
		$sliderID = (int) $this->configuration['sliderID_to_show'];

		if ( $sliderID > 0 )
		{
			try
			{
				$this->slider = \IPS\fcontent\Slider::loadAndCheckPerms( $sliderID, 'view' );			
			}
			catch( \OutOfRangeException $e ){}
	
			$cacheKey = $this->cacheKey;
			if ( $this->slider && isset( \IPS\Data\Store::i()->$cacheKey ) )
			{
				\IPS\Output::i()->endBodyCode .= \IPS\Theme::i()->getTemplate( 'embed', 'fcontent', 'front' )->sliderConfig( $this->slider );
			}
		}
	}
	
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		$form = parent::configuration( $form );

		$form->add( new \IPS\Helpers\Form\YesNo( 'fcontent_use_wrapper', $this->configuration['fcontent_use_wrapper'], TRUE, array() ) );	

		$options = array();
		
		foreach( \IPS\Db::i()->select( '*', 'fcontent_sliders', "fcs_enabled=1", 'fcs_id' ) as $sd )
		{
			$options[ $sd['fcs_id'] ]	= $sd['fcs_title'];
		}

		$form->add( new \IPS\Helpers\Form\Select( 'sliderID_to_show', isset( $this->configuration['sliderID_to_show'] ) ? \intval( $this->configuration['sliderID_to_show'] ) : 0, TRUE, array( 'options' => $options, 'parse' => 'normal' ) ) );	
		
		$form->add( new \IPS\Helpers\Form\Radio( 'fcontent_where_to_show', $this->configuration['fcontent_where_to_show'], TRUE, array(
			'options' => array( 'all' => 'fcontent_where_to_show_all', 'pages' => 'fcontent_where_to_show_pages' ),
			'toggles' => array(
				'pages'	=> array( 'fcontent_pages' )
			)
		) ) );		
		
		$form->add( new \IPS\Helpers\Form\TextArea( 'fcontent_pages', isset( $this->configuration['fcontent_pages'] ) ? $this->configuration['fcontent_pages'] : $_SERVER["HTTP_REFERER"], FALSE, array('rows' => 3), null, null, null, 'fcontent_pages' ) );	
				
 		return $form;
 	} 
 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{	
 		return $values;
 	}
	
	/**
	 * Convert the widget to HTML
	 *
	 * @return	string
	 */
	public function __toString()
	{
		if ( $this->configuration['fcontent_where_to_show'] == 'pages' && $this->configuration['fcontent_pages'])
		{
			$currentpageURL = $_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
			$pages = explode("\n", str_replace("\r", '', $this->configuration['fcontent_pages']));
			foreach( $pages as $page )
			{
				$page = explode("://", $page);
				$pagelist[] = $page[1];
			}
			if ( ! \in_array( $currentpageURL, $pagelist ) )
			{
				return '';
			}
		}
		
		return parent::__toString();
	}
	
	public function render()
	{
		if ( $this->slider )
		{
			$content = $this->slider->getContents();
			return $this->output( $this->slider, $content, $this->configuration );						
		}
		
		return '';
	}	
}